package cnv.hf.widgets;

import java.util.HashMap;

import android.content.Context;
import android.graphics.Color;
import android.graphics.drawable.Drawable;
import android.support.v4.view.PagerAdapter;
import android.support.v4.view.ViewPager;
import android.util.AttributeSet;
import android.view.Gravity;
import android.view.View;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import cnv.hf.widgets.HFWidgetStorage.HFPageControlStorage;

public class HFViewPagerWidget extends HFBaseWidget {

	private HashMap<Integer, HFLayerWidget> mLayersList; // ͼ
	private int mPageNum;
	private int[] mPageLayersIDs;
	private int[] mPageIndexsMapping;
	private HFViewPagerAdapterWidget mViewPagerAdapterWidget;
	private ViewPagerLayout mViewPagerLayout;
	private Drawable mImgDrawable;
	private Drawable mImgSelectedDrawable;
	private int mImgMarginLeft;
	private int mImgMarginRight;
	private int mImgMarginTop = 0;
	private int mImgMarginBottom = 0;
	private int mImgWidth = 0;
	private int mImgHeight = 0;
	
	public HFViewPagerWidget(Context context, Object widgetData) {
		super(context, widgetData);
		// TODO Auto-generated constructor stub
		initViewPagerMembers(widgetData);
	}

	protected void setHolder(Context context, Object widgetData) {
		// setObject(new ViewPager(context));
		mViewPagerLayout = new ViewPagerLayout(context);
		setObject(mViewPagerLayout);
	}

	private void initViewPagerMembers(Object widgetData) {
		HFPageControlStorage pData = (HFPageControlStorage) widgetData;
		if (pData != null) {
			mLayersList = new HashMap<Integer, HFLayerWidget>();
			mPageNum = pData.getNumOfPages();
			if (mPageNum > 0) {
				mPageLayersIDs = null;
				mPageLayersIDs = new int[mPageNum];
				for (int i = 0; i < mPageNum; i++)
					mPageLayersIDs[i] = pData.getPageLayerIds()[i];
			}

		}
		mImgMarginLeft = (int) (3 * HFModesManager.getCurrentMode().getXScaleFactor());
		mImgMarginRight = mImgMarginLeft;
		
		mImgWidth = (int) (16* HFModesManager.getCurrentMode().getXScaleFactor());
		mImgHeight = mImgWidth;
	}

	public void setGuideDrawable(Drawable drawable) {
		mImgDrawable = drawable;
	}

	public void setGuideDrawable(int drawable) {
		mImgDrawable = mViewPagerLayout.getContext().getResources()
				.getDrawable(drawable);
	}

	public void setGuideSelectedDrawable(Drawable drawable) {
		mImgSelectedDrawable = drawable;
	}

	public void setGuideSelectedDrawable(int drawable) {
		mImgSelectedDrawable = mViewPagerLayout.getContext().getResources()
				.getDrawable(drawable);
	}

	public void setAdapter(HFViewPagerAdapterWidget adapter) {
		mViewPagerAdapterWidget = adapter;
		if (adapter != null
				&& (((ViewPagerLayout) getObject()).getViewPager())
						.getAdapter() == null) {
			(((ViewPagerLayout) getObject()).getViewPager())
					.setAdapter(new HFViewPagerAdapter());
			mViewPagerLayout.addImgs(adapter.getCount());
		}
		if (adapter == null)
			(((ViewPagerLayout) getObject()).getViewPager()).setAdapter(null);
	}

	public interface HFViewPagerAdapterWidget {
		public int getCount();

		public Object instantiateItem(View page, int index);
	}

	/*
	 * ====================================================== : ݸıʱ,֪ͨˢ
	 * : ֵ:  ======================================================
	 */
	public void notifyDataChanged() {
		PagerAdapter adapter = (((ViewPagerLayout) getObject()).getViewPager())
				.getAdapter();
		if (adapter != null) {
			mViewPagerLayout.addImgs(adapter.getCount());
			adapter.notifyDataSetChanged();
		}
		getObject().postInvalidate();
	}

	/*
	 * ====================================================== : ͨͼIdȡͼ
	 * : Id [in] ͼId
	 * 
	 * ֵ: ͼ ======================================================
	 */
	protected HFLayerWidget createLayerById(int layerId) {

		if (layerId <= -1) {
			return null;
		}
		View view = getObject();
		if (view == null) {
			return null;
		}
		HFModeWidget modeWidget = (HFModeWidget) (view.getContext());
		if (modeWidget != null) {
			return modeWidget.createLayerWidgets(layerId);
		}
		return null;
	}

	/**
	 * ܣָpage
	 * 
	 * @param itemIndex
	 */
	public void setCurrentItem(int itemIndex) {
		(((ViewPagerLayout) getObject()).getViewPager())
				.setCurrentItem(itemIndex);
		LinearLayout layout = mViewPagerLayout.getImgsLayout();
		if (layout != null) {
			int count = layout.getChildCount();
			ImageView imageView = null;
			for (int i = 0; i < count; i++) {
				if (layout.getChildAt(i) instanceof ImageView) {
					imageView = (ImageView) layout.getChildAt(i);
					if (mImgDrawable != null)
						imageView.setImageDrawable(mImgDrawable);
					if (mImgSelectedDrawable != null && i == itemIndex)
						imageView.setImageDrawable(mImgSelectedDrawable);
				}

			}
		}
		mViewPagerLayout.postInvalidate();
	}
	
	/** ͼmarginֵ
	 * @param marginLeft
	 * @param marginTop
	 * @param marginRight
	 * @param marginBottom
	 * @return void
	 * @author LiangYJ
	 * @date 2015-7-7 9:33:29
	 */ 
	public void setGuideImgsMargin(int marginLeft,int marginTop,int marginRight,int marginBottom){
		mImgMarginLeft = marginLeft;
		mImgMarginTop = marginTop;
		mImgMarginRight = marginRight;
		mImgMarginBottom = marginBottom;
	}
	
	/** ͼĴС
	 * @param width
	 * @param height
	 * @return void
	 * @author LiangYJ
	 * @date 2015-7-7 9:45:06
	 */ 
	public void setGuideImgsWidthHeight(int width,int height){
		mImgWidth = width;
		mImgHeight = height;
	}

	/**
	 * ܣȡǰҳ
	 * 
	 * @return
	 */
	public int getCurrentItem() {
		return (((ViewPagerLayout) getObject()).getViewPager())
				.getCurrentItem();
	}

	/**
	 * ܣItem
	 * 
	 * @param itemIndex
	 */
	public void resetItem(int itemIndex) {
		if (itemIndex < 0) {
			mLayersList.clear();
		} else {
			mLayersList.remove(itemIndex);
		}
	}

	public int[] getPageIndexsMapping() {
		return mPageIndexsMapping;
	}

	public void setPageIndexsMapping(int[] mPageIndexsMapping) {
		this.mPageIndexsMapping = mPageIndexsMapping;
	}

	private class HFViewPagerAdapter extends PagerAdapter {

		@Override
		public int getCount() {
			// TODO Auto-generated method stub
			int num = 0;
			if (mViewPagerAdapterWidget != null)
				num = mViewPagerAdapterWidget.getCount();
			if (num < 0)
				num = 0;
			else if (num == 0)
				num = mPageNum;
			return num;
		}

		@Override
		public boolean isViewFromObject(View arg0, Object arg1) {
			// TODO Auto-generated method stub
			return arg0 == arg1;
		}

		@Override
		public int getItemPosition(Object object) {
			// TODO Auto-generated method stub
			// return super.getItemPosition(object);
			return POSITION_NONE;
		}

		@Override
		public void destroyItem(View arg0, int arg1, Object arg2) {
			// TODO Auto-generated method stub
			if (mLayersList.get(arg1) != null)
				((ViewPager) arg0).removeView(mLayersList.get(arg1));
		}

		@Override
		public Object instantiateItem(View arg0, int arg1) {
			// TODO Auto-generated method stub
			int layID = 0; // ͼID
			if (mPageLayersIDs.length <= 0)
				return null;
			else if (mPageLayersIDs.length == 1)
				layID = mPageLayersIDs[0];
			else {
				if (mPageIndexsMapping != null) {
					layID = mPageLayersIDs[mPageIndexsMapping[arg1]];
				} else {
					layID = mPageLayersIDs[arg1];
				}
			}

			HFLayerWidget pageLay = mLayersList.get(arg1);

			if (pageLay == null) {
				pageLay = createLayerById(layID);
				mLayersList.put(arg1, pageLay);
			}

			if (mViewPagerAdapterWidget != null)
				mViewPagerAdapterWidget.instantiateItem(pageLay, arg1);
			((ViewPager) arg0).addView(pageLay);
			return pageLay;
		}

	}

	private class ViewPagerLayout extends RelativeLayout {

		private Context mContext;
		private LinearLayout mLayoutImgs = null;
		private ViewPager mViewPager;

		/**
		 * @param context
		 * @param attrs
		 * @param defStyle
		 */
		public ViewPagerLayout(Context context, AttributeSet attrs, int defStyle) {
			super(context, attrs, defStyle);
			// TODO Auto-generated constructor stub
			addViews(context);
		}

		/**
		 * @param context
		 * @param attrs
		 */
		public ViewPagerLayout(Context context, AttributeSet attrs) {
			super(context, attrs);
			// TODO Auto-generated constructor stub
			addViews(context);
		}

		/**
		 * @param context
		 */
		public ViewPagerLayout(Context context) {
			super(context);
			// TODO Auto-generated constructor stub
			addViews(context);
		}

		private void addViews(Context context) {
			mContext = context;
			mLayoutImgs = new LinearLayout(context);
			mViewPager = new ViewPager(context);
			LayoutParams lParams = new LayoutParams(LayoutParams.MATCH_PARENT,
					(int) (28 * HFModesManager.getCurrentMode().getXScaleFactor()));
			lParams.addRule(ALIGN_PARENT_TOP);
			lParams.topMargin = (int) (8 * HFModesManager.getCurrentMode().getXScaleFactor()) ;
			
			mLayoutImgs.setGravity(Gravity.CENTER);
			addView(mViewPager);
			addView(mLayoutImgs, lParams);

		}

		/** ע
		 * @param count
		 * @return void
		 * @author LiangYJ
		 * @date 2015-7-1 8:17:35
		 */ 
		public void addImgs(int count) {
			mLayoutImgs.removeAllViews();
			if (mImgDrawable == null)
				return;
			ImageView imageView = null;
			android.widget.LinearLayout.LayoutParams layoutParams = new android.widget.LinearLayout.LayoutParams(
					mImgHeight, mImgWidth);
			layoutParams.gravity = Gravity.CENTER_VERTICAL;
			layoutParams.leftMargin = mImgMarginLeft;
			layoutParams.rightMargin = mImgMarginRight;
			layoutParams.topMargin = mImgMarginTop;
			layoutParams.bottomMargin = mImgMarginBottom;
			int selected = getCurrentItem();
			for (int i = 0; i < count; i++) {
				imageView = new ImageView(mContext);
				if(selected == i && mImgSelectedDrawable != null)
				    imageView.setImageDrawable(mImgSelectedDrawable);
				else if(mImgDrawable != null)
					imageView.setImageDrawable(mImgDrawable);
				imageView.setLayoutParams(layoutParams);
				mLayoutImgs.addView(imageView);
			}
		}

		public ViewPager getViewPager() {
			return mViewPager;
		}

		public LinearLayout getImgsLayout() {
			return mLayoutImgs;
		}
	}

	/** ȡViewPager
	 * @return
	 * @return ViewPager
	 * @author LiangYJ
	 * @date 2015-9-10 5:15:15
	 */ 
	public ViewPager getViewPager() {
		return mViewPagerLayout.getViewPager();
	}
	
	/** ȡҳ㲼
	 * @return
	 * @return LinearLayout
	 * @author LiangYJ
	 * @date 2015-9-10 5:15:26
	 */ 
	public LinearLayout getImgsLayout(){
		return mViewPagerLayout.getImgsLayout();
	}

}
